<?php

namespace Drupal\translate_side_by_side\Form;

use Drupal\Core\Entity\EntityFieldManager;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Language\Language;
use Drupal\Core\Language\LanguageManager;
use Drupal\Core\Menu\MenuLinkTree;
use Drupal\Core\Menu\MenuTreeParameters;
use Drupal\Core\Render\Renderer;
use Drupal\taxonomy\Entity\Vocabulary;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a form to configure module settings.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * The variable containing the language manager.
   *
   * @var \Drupal\Core\Language\LanguageManager
   */
  protected $languageManager;

  /**
   * The variable containing the entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * The variable containing the entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected $entityFieldManager;

  /**
   * The variable containing the menu tree link.
   *
   * @var \Drupal\Core\Menu\MenuLinkTree
   */
  protected $menuLinkTree;

  /**
   * The variable containing the renderer.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * Dependency injection through the constructor.
   *
   * @param \Drupal\Core\Language\LanguageManager $languageManager
   *   The language service.
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   *   The entity type service.
   * @param \Drupal\Core\Entity\EntityFieldManager $entityFieldManager
   *   The entity field service.
   * @param \Drupal\Core\Menu\MenuLinkTree $menuLinkTree
   *   The menu link tree  service.
   * @param \Drupal\Core\Render\Renderer $renderer
   *   The renderer service.
   */
  public function __construct(LanguageManager $languageManager,
  EntityTypeManager $entityTypeManager,
  EntityFieldManager $entityFieldManager,
  MenuLinkTree $menuLinkTree,
  Renderer $renderer) {
    $this->languageManager = $languageManager;
    $this->entityTypeManager = $entityTypeManager;
    $this->entityFieldManager = $entityFieldManager;
    $this->menuLinkTree = $menuLinkTree;
    $this->renderer = $renderer;
  }

  /**
   * Dependency injection create.
   */
  public static function create(ContainerInterface $container) {
    return new static ($container->get('language_manager'),
    $container->get('entity_type.manager'),
    $container->get('entity_field.manager'),
    $container->get('menu.link_tree'),
    $container->get('renderer'));
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'translate_side_by_side.settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['config.translate_side_by_side'];
  }

  /**
   * Build form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form = [];

    $form['selection'] = [
      '#title' => $this->t('Source language: @name',
      ['@name' => $this->languageManager->getDefaultLanguage()->getName()]),
      '#type' => 'fieldset',
    ];
    $form['selection']['target'] = [
      '#title' => $this->t('Target language'),
      '#type' => 'language_select',
      '#languages' => Language::STATE_CONFIGURABLE,
    ];
    $form['selection']['filluntranslated'] = [
      '#title' => $this->t('Fill untranslated with source'),
      '#type' => 'checkbox',
    ];
    $form['selection']['load'] = [
      '#type' => 'button',
      '#value' => $this->t('Load'),
    ];

    $target = ($form_state->getValue('target') != '') ? ($form_state
      ->getValue('target')) : ($this->languageManager->getDefaultLanguage()
      ->getId());

    $form['menus'] = [
      '#value' => $this->t('Menus'),
      '#type' => 'html_tag',
      '#tag' => 'h3',
    ];
    $form = $this->buildFormMenu($form, $form_state, $target);

    $form['nodes'] = [
      '#value' => $this->t('Nodes'),
      '#type' => 'html_tag',
      '#tag' => 'h3',
    ];
    $form = $this->buildFormEntity($form, $form_state, 'node', 'title', 'nid', $target);

    $form['blocks'] = [
      '#value' => $this->t('Blocks'),
      '#type' => 'html_tag',
      '#tag' => 'h3',
    ];
    $form = $this->buildFormEntity($form, $form_state, 'block_content', 'info', 'id', $target);

    $form['taxonomy'] = [
      '#value' => $this->t('Taxonomy'),
      '#type' => 'html_tag',
      '#tag' => 'h3',
    ];
    $form = $this->buildFormTaxonomy($form, $form_state, $target);

    $form = parent::buildForm($form, $form_state);
    unset($form['actions']['submit']);
    return $form;
  }

  /**
   * Build entity form.
   */
  private function buildFormEntity(
  array $form,
  FormStateInterface $form_state,
  $entityType,
  $sortBy,
  $idKey,
  $target) {
    $nids = $this->entityTypeManager->getStorage($entityType)->getQuery()
      ->condition('langcode', 'zzz', '<>')
      ->condition('langcode', 'zxx', '<>')
      ->sort($sortBy, 'ASC', 'en')
      ->execute();
    $entities = $this->entityTypeManager->getStorage($entityType)->loadMultiple($nids);
    foreach ($entities as $oneEntity) {
      if ($oneEntity->hasTranslation($target)) {
        $translation = $oneEntity->getTranslation($target);
      }
      else {
        $translation = NULL;
      }

      $fieldlist = [];
      $fields = $oneEntity->getFieldDefinitions();

      foreach ($fields as $field_name => $field_definition) {

        $field_display = $this->entityTypeManager
          ->getStorage('entity_view_display')
          ->load($entityType . '.' . $oneEntity->bundle() . '.default')
          ->getComponent($field_name);

        $field_display_weight = ($field_display !== NULL) ? ($field_display['weight'] + 1) : (0);

        if (($field_name == 'title' || $field_name == 'body' || $field_name == 'info' ||
        substr($field_name, 0, 6) === 'field_')) {
          $fieldlist = $this->retrieveField($fieldlist, $field_definition, $translation, $field_name, $oneEntity, $field_display_weight, $target, $form_state);
        }
      }
      ksort($fieldlist);
      $form['tablet' . $entityType][] = $this->buildFormTable($form, $form_state, $target,
      $oneEntity->get($sortBy)->value . ' ' . str_replace('/' .
      $this->languageManager->getDefaultLanguage()->getId(), '',
      $oneEntity->toUrl()->toString()), $entityType . ' ID: ' . $oneEntity->get($idKey)->value,
      $oneEntity->language()->getName(), $fieldlist, $entityType . $oneEntity->get($idKey)->value);

    }
    return $form;
  }

  /**
   * Build field.
   */
  private function retrieveField($fieldlist, $field_definition, $translation, $field_name, $oneEntity, $field_display_weight, $target, $form_state) {
    foreach ($oneEntity->get($field_name)->getValue() as $onekey =>
      $onevalue) {

      $txtattr = NULL;

      if ($field_definition->getType() == 'image') {
        $txtattr = 'alt';
      }
      elseif ($field_definition->getType() == 'link') {
        $txtattr = 'title';
      }
      elseif (strpos($field_definition->getType(), 'string') !== FALSE
      || strpos($field_definition->getType(), 'text') !== FALSE) {
        $txtattr = 'value';
      }
      elseif ($field_definition->getType() === 'entity_reference_revisions'
      || $field_definition->getType() === 'field_collection') {
        foreach ($oneEntity->get($field_name)->referencedEntities() as $oneparkey => $oneparvalue) {
          if ($oneparvalue->hasTranslation($target)) {
            $par_translation = $oneparvalue->getTranslation($target);
          }
          else {
            $par_translation = NULL;
          }
          $par_fields = $oneparvalue->getFieldDefinitions();
          foreach ($par_fields as $par_field_name => $par_field_definition) {
            if (substr($par_field_name, 0, 6) === 'field_') {
              $par_field_display = $this->entityTypeManager
                ->getStorage('entity_view_display')
                ->load($oneEntity->get($field_name)->getFieldDefinition()->getSetting('target_type') . '.' . $oneparvalue->bundle() . '.default')
                ->getComponent($par_field_name);
              $par_field_display_weight = $field_display_weight + $oneparkey * 10 + (($par_field_display !== NULL) ? ($par_field_display['weight'] + 1) : (0));

              $fieldlist = $this->retrieveField($fieldlist, $par_field_definition, $par_translation, $par_field_name, $oneparvalue, $par_field_display_weight, $target, $form_state);
            }
          }
        }
        return $fieldlist;
      }
      /* Nothing to translate e.g. entity_reference */
      if ($txtattr === NULL) {
        return $fieldlist;
      }

      if ($translation !== NULL) {
        $field_val = $translation->get($field_name)
          ->getValue()[$onekey][$txtattr];
      }
      elseif ($form_state->getValue('filluntranslated') === 0) {
        $field_val = '';
      }
      else {
        $field_val = $onevalue[$txtattr];
      }

      if ($txtattr !== NULL) {
        $fieldlist[$field_display_weight * 1000 + $onekey] = [
          'f' => $field_name . (($txtattr !== 'value') ? (' (' . $txtattr . ')') : ('')),
          's' => $onevalue[$txtattr],
          't' => $field_val,
        ];
      }
    }
    return $fieldlist;
  }

  /**
   * Build menu form.
   */
  private function buildFormMenu(array $form, FormStateInterface $form_state, $target) {
    $mids = $this->entityTypeManager->getStorage('menu')->getQuery()
      ->condition('langcode', 'zzz', '<>')
      ->condition('langcode', 'zxx', '<>')
      ->condition('id', 'account', '<>')
      ->condition('id', 'admin', '<>')
      ->condition('id', 'devel', '<>')
      ->condition('id', 'tools', '<>')
      ->sort('label', 'ASC', 'en')
      ->execute();
    $menus = $this->entityTypeManager->getStorage('menu')->loadMultiple($mids);

    foreach ($menus as $onemenu) {

      $fieldlist = [];

      $parameters = new MenuTreeParameters();
      $parameters->setMinDepth(0)->setMaxDepth(4)->onlyEnabledLinks();
      $tree = $this->menuLinkTree->load($onemenu->get('id'), $parameters);
      $manipulators = [
        ['callable' => 'menu.default_tree_manipulators:generateIndexAndSort'],
        ['callable' => 'menu.default_tree_manipulators:flatten'],
      ];
      $tree = $this->menuLinkTree->transform($tree, $manipulators);
      foreach ($tree as $element) {
        $linkDef = $element->link->getPluginDefinition();

        /* empty entity_id */
        if (empty($linkDef['metadata']['entity_id'])) {
          continue;
        }

        $entity_id = $linkDef['metadata']['entity_id'];

        $storage = $this->entityTypeManager
          ->getStorage('menu_link_content');

        $entity = $storage->load($entity_id);

        if ($entity->isTranslatable() && $entity->hasTranslation($target)) {
          $translation = $entity->getTranslation($target);
        }
        else {
          $translation = NULL;
        }
        if ($translation !== NULL) {
          $menu_val = $entity->getTranslation($target)->getTitle() . '<br>' . $entity->getTranslation($target)->getDescription();
        }
        elseif ($form_state->getValue('filluntranslated') === 0) {
          $menu_val = '';
        }
        else {
          $menu_val = $entity->getTitle() . '<br>' . $entity->getDescription();
        }

        $fieldlist[] = [
          'f' => $entity_id,
          's' => $entity->getTitle(),
          't' => $menu_val,
        ];
      }

      $form['tabletmenu'][] = $this->buildFormTable($form, $form_state, $target,
      $onemenu->get('label'), 'machine name: ' . $onemenu->get('id'),
      $onemenu->language()->getName(), $fieldlist, 'menu' . $onemenu->get('id'));
    }
    return $form;
  }

  /**
   * Build taxonomy form.
   */
  private function buildFormTaxonomy(array $form, FormStateInterface $form_state, $target) {
    $vocabularies = Vocabulary::loadMultiple();

    foreach ($vocabularies as $onevocabulary) {
      $tids = $this->entityTypeManager->getStorage('taxonomy_term')->getQuery()
        ->condition('vid', $onevocabulary->get('vid'))
        ->execute();
      $terms = $this->entityTypeManager->getStorage('taxonomy_term')->loadMultiple($tids);

      foreach ($terms as $oneterm) {
        $fieldlist = [];
        if ($oneterm->isTranslatable() && $oneterm->hasTranslation($target)) {
          $translation = $oneterm->getTranslation($target);
        }
        else {
          $translation = NULL;
        }
        if ($translation !== NULL) {
          $name_val = $oneterm->getTranslation($target)->getName();
          $desc_val = $oneterm->getTranslation($target)->getDescription();
        }
        elseif ($form_state->getValue('filluntranslated') === 0) {
          $name_val = '';
          $desc_val = '';
        }
        else {
          $name_val = $oneterm->getName();
          $desc_val = $oneterm->getDescription();
        }

        $fieldlist[] = [
          'f' => 'name',
          's' => $oneterm->getName(),
          't' => $name_val,
        ];
        $fieldlist[] = [
          'f' => 'description',
          's' => $oneterm->getDescription(),
          't' => $desc_val,
        ];

        $form['tablettax'][] = $this->buildFormTable($form, $form_state, $target,
        $onevocabulary->get('name'), 'machine name: ' . $onevocabulary->get('vid') . ', ID: ' . $oneterm->id(),
        $onevocabulary->language()->getName(), $fieldlist, 'taxonomy' . $onevocabulary->get('vid') . $oneterm->id());

      }

    }
    return $form;
  }

  /**
   * Build form table for all types above.
   */
  private function buildFormTable(array $form,
  FormStateInterface $form_state,
  $target,
  $name1,
  $name2,
  $sourcelang,
  array $fieldlist,
  $entityType) {
    $table = [
      '#type' => 'table',
      '#caption' => [
        '#markup' => '<strong>' . $name1 . ':</strong>  <small>(' . $name2 . ')</small>',
        '#attributes' => ['align' => 'left'],
      ],
      '#header' => [
        ['data' => 'ID', 'width' => '20%'],
        ['data' => $sourcelang, 'width' => '40%'],
        [
          'data' => $this->languageManager->getLanguage($target)->getName(),
          'width' => '40%',
        ],
      ],
      '#attributes' => [
        'border' => '1',
        'width' => '100%',
        'class' => ['tsbstable', 'tsbstable_' . $entityType],
      ],
    ];
    foreach ($fieldlist as $k => $onefieldlist) {
      $table['#rows'][$k]['valign'] = 'top';
      $table['#rows'][$k]['class'][] = 'tsbstable_' . $entityType . '_' . $onefieldlist['f'];

      $row = [];

      $r1html = [];
      $r1html['#markup'] = '<small>' . $onefieldlist['f'] . '</small>';
      $row[] = ['data' => $this->renderer->render($r1html)];

      $r2html = [];
      $r2html['#markup'] = $onefieldlist['s'];
      $row[] = [
        'data' => $this->renderer->render($r2html),
        'lang' => $this->languageManager->getDefaultLanguage()->getId(),
      ];

      $r3html = [];
      $r3html['#markup'] = $onefieldlist['t'];
      $row[] = [
        'data' => $this->renderer->render($r3html),
        'lang' => $target,
      ];

      $table['#rows'][$k]['data'] = $row;
    }
    $formtable['table'] = $table;
    $formtable['br'] = [
      '#type' => 'html_tag',
      '#tag' => 'br',
    ];
    return $formtable;
  }

}
